cordova.define("simdoll-storage.SDLocalStorageFileBackup", function(require, exports, module) {

var channel = require('cordova/channel');
var fileName = "cord_game_file";
var fileWriter = null;

var isWriting = false; // 是否正在写入数据
var storageDirty = false;

function hookLocalStorage() {
    var storageObj = window.localStorage;
    var proto = Object.getPrototypeOf(storageObj);
    var originalSetItem = proto.setItem;
    var originalRemoveItem = proto.removeItem;
    var originalClear = proto.clear;
    proto.setItem = function (key, newValue) {
        originalSetItem.apply(this, arguments);
        if (this === storageObj && !filterOutKey(key)) {  // ensure only localStorage get processed(session storage will not trigger backup)
            triggerBackup();
        }
    }
    proto.removeItem = function (key) {
        originalRemoveItem.apply(this, arguments);
        if (this === storageObj && !filterOutKey(key)) {
            triggerBackup();
        }
    }
    proto.clear = function () {
        originalClear.apply(this, arguments);
        if (this === storageObj) {
            triggerBackup();
        }
    }
}


function filterOutKey(k) {
    return k.indexOf('cache') >= 0
}

function triggerBackup() {
    storageDirty = true;
    if (!isWriting) {
        isWriting = true;
        setTimeout(beginBackup, 1000);    // a bit delay to reduce write
    }
}
function beginBackup() {
    var dataObj = {};
    for (var i = 0; i < window.localStorage.length; ++i) {
        var k = window.localStorage.key(i);
        if (!filterOutKey(k)) {   // filter out cache
            var v = window.localStorage.getItem(k);
            dataObj[k] = v;
        }
    }
    storageDirty = false;
    var jsonStr = JSON.stringify(dataObj);
    fileWriter.onwrite = function () {
        //console.log("Successful file write..." + Math.floor(Date.now()/1000));
        endBackup(true);
    };
    fileWriter.onerror = function (e) {
        endBackup(false);
    };
    fileWriter.seek(0); // overwrite from begging
    fileWriter.write(jsonStr);  // write can also truncate file to the size of data
}
function endBackup(succeeded) {
    if (storageDirty) {
        setTimeout(beginBackup, 1000);    // a bit delay to reduce write
    } else {
        isWriting = false;
    }
}

function _prepareCallback(callback) {
    getBackupStorage(fileName, function () {
        callback();
    }, function (error) {
        console.error(error);
        callback();
    });
}
// 获取文件系统中的数据
function getBackupStorage(fileName, onSuccess, onError) {
    window.resolveLocalFileSystemURL(window.cordova.file.dataDirectory, function (dirEntry) {
        getBackupFile(dirEntry, fileName, onSuccess, onError);
    }, function (error) {
        console.error(error);
        onError(error);
    });
}
// 创建文件标识
function getBackupFile(dirEntry, fileName, onSuccess, onError) {
    var self = this;
    dirEntry.getFile(fileName, { create: true, exclusive: false }, function (fileEntry) {
        if (0 == window.localStorage.length) {
            loadBackupDataToMemory(fileEntry, function () {
                createWriter(fileEntry, onSuccess, onError);
            });
        } else {
            createWriter(fileEntry, onSuccess, onError);
        }
    }, function (error) {
        console.error(error);
        onError(error);
    });
}
function createWriter(fileEntry, onSuccess, onError) {
    fileEntry.createWriter(function (w) {
        fileWriter = w;
        if (null != fileWriter) {
            onSuccess();
        } else {
            onError(new Error("create writer failed"));
        }
    })
}
function loadBackupDataToMemory(fileEntry, onComplete) {
    fileEntry.file(function (file) {
        var reader = new FileReader();
        reader.onload = function () {
            //console.log("Successful file read: " + this.result);
            try {
                if (this.result && this.result.length > 0) {
                    var loadedData = JSON.parse(this.result);
                    if ("object" == typeof loadedData) {
                        for (var k in loadedData) {
                            if (null === localStorage.getItem(k)) {
                                localStorage.setItem(k, loadedData[k]);
                            }
                        }
                    }
                }
                onComplete();
            } catch (ex) { // json error, considered as success
                onComplete();
            }
        };
        reader.onerror = function (err) {
            onComplete();
        }
        reader.readAsText(file);
    });
}

channel.waitForInitialization('onStoragePluginReady');
channel.onCordovaReady.subscribe(function () {
    channel.onFileSystemPathsReady.subscribe(function () {
        _prepareCallback(function () {
            if (null != fileWriter)
                hookLocalStorage();
            channel.initializationComplete("onStoragePluginReady");
        });
    }
    );
})

});
